/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "dusk.h"

typedef uint32_t fixed248_t;

#define FIXED248_FRACTION_BITS 8
#define FIXED248_HIGH_MULTIPLIER (1 << FIXED248_FRACTION_BITS)
#define FIXED248(i, f) ((fixed248_t)( \
  ((i) << FIXED248_FRACTION_BITS) + \
  (((f) * FIXED248_HIGH_MULTIPLIER) / 100) \
))

/**
 * Convert an int32_t value to a fixed248_t value.
 * 
 * @param b The int32_t value to convert.
 * @return The converted fixed248_t value.
 */
fixed248_t fx248Fromi32(const int32_t b);

/**
 * Convert a uint32_t value to a fixed248_t value.
 * 
 * @param b The uint32_t value to convert.
 * @return The converted fixed248_t value.
 */
fixed248_t fx248Fromui32(const uint32_t b);

/**
 * Convert a float_t value to a fixed248_t value.
 * 
 * @param b The float_t value to convert.
 * @return The converted fixed248_t value.
 */
fixed248_t fx248Fromf32(const float_t b);

/**
 * Convert a fixed248_t value to an int32_t value.
 * 
 * @param a The fixed248_t value to convert.
 * @return The converted int32_t value.
 */
int32_t fx248Toi32(const fixed248_t a);

/**
 * Convert a fixed248_t value to a uint32_t value.
 * 
 * @param a The fixed248_t value to convert.
 * @return The converted uint32_t value.
 */
uint32_t fx248Tou32(const fixed248_t a);

/**
 * Convert a fixed248_t value to a float_t value.
 * 
 * @param a The fixed248_t value to convert.
 * @return The converted float_t value.
 */
float_t fx248Tof32(const fixed248_t a);

/**
 * Add a fixed248_t value to another fixed248_t value.
 * 
 * @param a Pointer to the first fixed248_t value (will be modified).
 * @param b The fixed248_t value to add to the first value.
 */
void fx248Addfx248(fixed248_t *a, const fixed248_t b);

/**
 * Add an int32_t value to a fixed248_t value.
 * 
 * @param a Pointer to the fixed248_t value (will be modified).
 * @param b The int32_t value to add to the fixed248_t value.
 */
void fx248Addi32(fixed248_t *a, const int32_t b);

/**
 * Add a uint32_t value to a fixed248_t value.
 * 
 * @param a Pointer to the fixed248_t value (will be modified).
 * @param b The uint32_t value to add to the fixed248_t value.
 */
void fx248Addui32(fixed248_t *a, const uint32_t b);

/**
 * Add a float_t value to a fixed248_t value.
 * 
 * @param a Pointer to the fixed248_t value (will be modified).
 * @param b The float_t value to add to the fixed248_t value.
 */
void fx248Addf32(fixed248_t *a, const float_t b);

/**
 * Subtract a fixed248_t value from another fixed248_t value.
 * 
 * @param a Pointer to the first fixed248_t value (will be modified).
 * @param b The fixed248_t value to subtract from the first value.
 */
void fx248Subfx248(fixed248_t *a, const fixed248_t b);

/**
 * Subtract an int32_t value from a fixed248_t value.
 * 
 * @param a Pointer to the fixed248_t value (will be modified).
 * @param b The int32_t value to subtract from the fixed248_t value.
 */
void fx248Subi32(fixed248_t *a, const int32_t b);

/**
 * Subtract a uint32_t value from a fixed248_t value.
 * 
 * @param a Pointer to the fixed248_t value (will be modified).
 * @param b The uint32_t value to subtract from the fixed248_t value.
 */
void fx248Subui32(fixed248_t *a, const uint32_t b);

/**
 * Subtract a float_t value from a fixed248_t value.
 * 
 * @param a Pointer to the fixed248_t value (will be modified).
 * @param b The float_t value to subtract from the fixed248_t value.
 */
void fx248Subf32(fixed248_t *a, const float_t b);

/**
 * Convert a fixed248_t value to an int32_t value, rounding towards zero.
 * 
 * @param a The fixed248_t value to convert.
 * @return The converted int32_t value.
 */
int32_t fx248Floor(const fixed248_t a);

/**
 * Convert a fixed248_t value to an int32_t value, rounding towards positive
 * infinity.
 * 
 * @param a The fixed248_t value to convert.
 * @return The converted int32_t value.
 */
int32_t fx248Ceil(const fixed248_t a);

/**
 * Convert a fixed248_t value to an int32_t value, rounding to the nearest
 * integer.
 * 
 * @param a The fixed248_t value to convert.
 * @return The converted int32_t value.
 */
int32_t fx248Round(const fixed248_t a);

/**
 * Convert a fixed248_t value to a uint32_t value, rounding towards zero.
 * 
 * @param a The fixed248_t value to convert.
 * @return The converted uint32_t value.
 */
uint32_t fx248Flooru32(const fixed248_t a);

/**
 * Convert a fixed248_t value to a uint32_t value, rounding towards positive
 * infinity.
 * 
 * @param a The fixed248_t value to convert.
 * @return The converted uint32_t value.
 */
uint32_t fx248Ceilu32(const fixed248_t a);

/**
 * Convert a fixed248_t value to a uint32_t value, rounding to the nearest
 * integer.
 * 
 * @param a The fixed248_t value to convert.
 * @return The converted uint32_t value.
 */
uint32_t fx248Roundu32(const fixed248_t a);